"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("apollo-codegen-core/lib/polyfills");
const command_1 = require("@oclif/command");
const Listr = require("listr");
const fs_1 = require("fs");
const commonTasks_1 = require("../../helpers/commands/queries/commonTasks");
const engine_cli_1 = require("../../engine-cli");
const load_config_1 = require("../../load-config");
class ExtractQueries extends command_1.Command {
    async run() {
        let { flags, args } = this.parse(ExtractQueries);
        const defaultFlags = { addTypename: true, removeClientDirectives: true };
        flags = Object.assign({}, defaultFlags, flags);
        const tasks = new Listr([
            load_config_1.loadConfigStep(flags, false),
            ...commonTasks_1.getCommonTasks({ flags, errorLogger: this.error.bind(this) }),
            ...commonTasks_1.getCommonManifestTasks({ flags }),
            {
                title: "Outputing extracted queries",
                task: (ctx, task) => {
                    const filename = args.output;
                    task.title = "Outputing extracted queries to " + filename;
                    fs_1.writeFileSync(filename, JSON.stringify(ctx.manifest, null, 2));
                }
            }
        ]);
        return tasks.run();
    }
}
ExtractQueries.description = "Extracts queries";
ExtractQueries.flags = Object.assign({ help: command_1.flags.help({
        char: "h",
        description: "Show command help"
    }), config: command_1.flags.string({
        description: "Path to your Apollo config file"
    }), queries: command_1.flags.string({
        description: "Path to your GraphQL queries, can include search tokens like **"
    }), addTypename: command_1.flags.boolean({
        description: "Automatically add __typename to your queries",
        allowNo: true
    }), removeClientDirectives: command_1.flags.boolean({
        allowNo: true,
        description: "Automatically remove @client and @connection directives and fields from operations"
    }) }, engine_cli_1.engineFlags, { tagName: command_1.flags.string({
        description: "Name of the template literal tag used to identify template literals containing GraphQL queries in Javascript/Typescript code",
        default: "gql"
    }) });
ExtractQueries.args = [
    {
        name: "output",
        description: "Path to write the extracted queries to",
        required: true,
        default: "manifest.json"
    }
];
exports.default = ExtractQueries;
//# sourceMappingURL=extract.js.map