"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const fs = require("fs");
const ci = require("env-ci");
const git_parse_1 = require("git-parse");
const git = require("git-rev-sync");
const lodash_1 = require("lodash");
const findGitRoot = (start) => {
    start = start || process.cwd();
    if (typeof start === "string") {
        if (start[start.length - 1] !== path.sep)
            start += path.sep;
        start = start.split(path.sep);
    }
    if (!start.length)
        return;
    start.pop();
    const dir = start.join(path.sep);
    if (fs.existsSync(path.join(dir, ".git"))) {
        return path.normalize(dir);
    }
    else {
        return findGitRoot(start);
    }
};
exports.gitInfo = async () => {
    const { isCi, commit, slug, root } = ci();
    const gitLoc = root ? root : findGitRoot();
    if (!commit)
        return;
    let committer;
    let remoteUrl = slug;
    if (gitLoc) {
        const { authorName, authorEmail } = await git_parse_1.gitToJs(gitLoc)
            .then((commits) => commits && commits.length > 0
            ? commits[0]
            : { authorName: null, authorEmail: null })
            .catch(() => ({ authorEmail: null, authorName: null }));
        committer = `${authorName || ""} ${authorEmail ? `<${authorEmail}>` : ""}`.trim();
        if (!isCi) {
            try {
                remoteUrl = git.remoteUrl();
            }
            catch (e) { }
        }
    }
    return lodash_1.pickBy({ committer, commit, remoteUrl }, lodash_1.identity);
};
//# sourceMappingURL=git.js.map